#    Copyright (C) 2000 Nick Feamster, Kevin Fu, Emil Sit, Kendra Smith
# 
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#  3. Neither the name of the University nor the names of its contributors
#     may be used to endorse or promote products derived from this software
#     without specific prior written permission.
# 
#  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#  SUCH DAMAGE.
# 

#!/usr/bin/perl

# Nick Feamster
# cookie-eaters
# January 2001

# basic_client.pl: This is a script to hammer a website with basic auth
# requests...

use LWP::UserAgent;
use HTTP::Request;
use HTTP::Response;
use HTTP::Cookies;
use LWP::Authen::Basic;

use Getopt::Long;

GetOptions(\%options, "get", "url=s", "loop=s");

my $cookie_file = "cookies.txt";

# here we hit the restricted site on the directly connected box: 4.0.0.2
# aka cookies.lcs.mit.edu

$url = "http://4.0.0.2/restricted/";
print "URL: " . $url;


#############################################################
# Process options
# --loop <n> authenticates to the server n successive times

if (exists $options{'loop'}) {
    $loop = $options{'loop'};
} else {
    $loop = 1;
}

# where to store the cookies which are retrieved, and where to
# load them up from


$cookie_jar = HTTP::Cookies::Netscape->new(File     => $cookie_file,
					   AutoSave => 1,
					   ignore_discard => 1);

# create an LWP user agent

my $ua = LWP::UserAgent->new();
$ua->agent("Cookie-Eaters/1.0 [en]");
$ua->from("user\@host.com");
#$ua->cookie_jar($cookie_jar);

$ua->credentials('4.0.0.2', 'cookierealm',
		 'user', 'password');

# ------------------------------------------------------------------ #
# Set up the Request

my $request = HTTP::Request->new(GET => $url);
$request->referer("http://referrer.host.com");
$request->content_type('text/html');
$request->authorization_basic('user', 'password');

# Pass our cookie(s) to the server
$cookie_jar->add_cookie_header($request);

# ------------------------------------------------------------------ #
# Set up the Response

$start_time = time();

for ($i=0; $i<$loop; $i++) {
    print "\nsending request $i...\n";
    $response = $ua->request($request);

##############################################################
# note that this section where potential set-cookie options are
# processed is not going to do anything, but is kept for consistency
# in tests to ensure that the basic_client.pl and client.pl perform
# as close as possible to similar behavior (for comparison).

# Extract the Set-cookie headers from the server response
    $cookie_jar->extract_cookies($response);

# Save the cookie jar's state
    $cookie_jar->save( $cookie_file );
###############################################################

    if ($response->is_error()) {
	printf ("\nThere was an error requesting %s ...\nStatus line: %s\n\n\a",
		$url, $response->status_line());
    } elsif ($loop==1) {
	printf ("\nSuccessfully retrieved %s !\n", $url);
	print $response->as_string();
	printf ("\nHere is the state of your cookie jar:\n\n%s",
		$cookie_jar->as_string());
    }

}


$end_time = time();
printf ("\n%d request(s) completed in %d seconds (%f sec/req)\n",
	$loop, $end_time-$start_time, ($end_time-$start_time)/$loop);
