#    Copyright (C) 2000 Nick Feamster, Kevin Fu, Emil Sit, Kendra Smith
# 
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#  3. Neither the name of the University nor the names of its contributors
#     may be used to endorse or promote products derived from this software
#     without specific prior written permission.
# 
#  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#  SUCH DAMAGE.
# 

#!/usr/bin/perl

# Nick Feamster
# cookie-eaters
# January 2001

# client.pl: fetches an anon auth cookie (as described in paper section 5)
# and represents it (in a loop) to the server to reauthenticate.

use LWP::UserAgent;
use HTTP::Request;
use HTTP::Response;
use HTTP::Cookies;

use Getopt::Long;

GetOptions(\%options, "get", "url=s", "loop=s");

my $cookie_file = "cookies.txt";


############################################
# Process Options
# --get gets the cookie from the server rather than trying to authenticate
# --loop <n> authenticates to the server n successive times

# note that "cookies.lcs.mit.edu" ==> 4.0.0.2 in our /etc/hosts file.
# this hack had to be made so that cookie domains worked for the script...

#  if ($options{'get'}) {
#      $url = "http://cookies.lcs.mit.edu/foo/directory/auth1.cgi";
#  } elsif (exists $options{'url'}) {
#      $url = $options{'url'};
#  } else {
#      $url = "http://cookies.lcs.mit.edu/foo/directory/login1.fcgi";
#  }

if (exists $options{'url'}) {
    $url = $options{'url'};
} else {
    print "ERROR: must supply URL!\n";
    exit(0);
}
print "URL: " . $url;

if (exists $options{'loop'}) {
    $loop = $options{'loop'};
} else {
    $loop = 1;
}

# where to store the cookies which are retrieved, and where to
# load them up from

$cookie_jar = HTTP::Cookies::Netscape->new(File     => $cookie_file,
					   AutoSave => 1,
					   ignore_discard => 1);

# create an LWP user agent

my $ua = LWP::UserAgent->new();
$ua->agent("Mozilla/4.75 [en]");
$ua->from("user\@host.com");
#$ua->cookie_jar($cookie_jar);



# this is useless for the cookie scheme, only relevant for
# basic auth test.  should remove...

$ua->credentials('4.0.0.2', 'cookierealm',
		 'feamster', 'password');

# ------------------------------------------------------------------ #
# Set up the Request

my $request = HTTP::Request->new(GET => $url);
$request->referer("http://lucky");
$request->content_type('text/html');

# Pass our cookie(s) to the server
$cookie_jar->add_cookie_header($request);

# ------------------------------------------------------------------ #
# Set up the Response

$start_time = time();

for ($i=0; $i<$loop; $i++) {
    print "\nsending request $i...\n";
    $response = $ua->request($request);


# Extract the Set-cookie headers from the server response
    $cookie_jar->extract_cookies($response);

# Save the cookie jar's state
    $cookie_jar->save( $cookie_file );

    if ($response->is_error()) {
	printf ("\nThere was an error requesting %s ...\nStatus line: %s\n\n\a",
		$url, $response->status_line());
    } elsif ($loop==1) {
	printf ("\nSuccessfully retrieved %s !\n", $url);
	print $response->as_string();
	printf ("\nHere is the state of your cookie jar:\n\n%s",
		$cookie_jar->as_string());
    }

}


$end_time = time();
printf ("\n%d request(s) completed in %d seconds (%f sec/req)\n",
	$loop, $end_time-$start_time, ($end_time-$start_time)/$loop);
