#    Copyright (C) 2000 Nick Feamster, Kevin Fu, Emil Sit, Kendra Smith
# 
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#  3. Neither the name of the University nor the names of its contributors
#     may be used to endorse or promote products derived from this software
#     without specific prior written permission.
# 
#  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#  SUCH DAMAGE.
# 

#!/usr/bin/perl

# Nick Feamster
# cookie-eaters
# January 2001

# login1.fcgi: This script is "the meat" of the benchmark.  It receives the 
# anonymous authentication cookie from the client, and matches the HMAC hash
# presented with the HMAC hash it computes on the expiration time (epoch)
# which is presented in the cookie along with the previously given HMAC.

use CGI;
use CGI::Cookie;	
use FCGI;

use Time::gmtime;
use Digest::HMAC;
use Digest::HMAC_MD5;
use Digest::HMAC_SHA1;
use Benchmark;

# this is the function used to check the digest.
sub checkdigest {

    $q = new CGI;
    
    my %cookies = fetch CGI::Cookie;
    print $q->header(-type=>'text/html');

    # if the client does not present a cookie, redirect him to the page
    # where an initial, password-based login occurs and where he can
    # get an anon auth cookie.

    if (!(exists $cookies{'expauth'})) {
	print "<meta http-equiv=\"Refresh\" content=\"1; url=http://your.host.here.com/auth1.html\">";
	print "<p>Authentication failed! => missing cookie<br>\n";
	return -1;
#	exit(0);
    }

    ######################################################
    # Parse the auth cookie

    $eastring = $cookies{'expauth'}->value();
    $eastring =~ s/%(..)/pack("c",hex($1))/ge;
    @pairs = split(/\&/, $eastring);
    
    foreach $pair (@pairs) {
	($key, $value) = split(/=/, $pair);
	$keys{$key} = $value;
    }
    
    $adigest = $keys{'digest'};
    $expdate = $keys{'exp'};
    
    #####################################################
    # compute the digest on the presented expiration date and
    # see if it matches the one we compute in real-time

    $key=0;  # again, this is the secret key.  make non-guessable. state??
    $hmac = Digest::HMAC_SHA1->new($key); # Change to HMAC_MD5 if necessary...
    $hmac->add($expdate);
    $digest = $hmac->hexdigest;

    return 0;
}


#################################
# The MAIN FCGI loop... very easy to do FCGI, just put your code into one
# of these loops.  This code also runs its own little mini benchmark.

while (FCGI::accept >= 0) {

    $t0 = new Benchmark;
    $res = checkdigest();
    if ($res < 0) {
	next;
    }
    $t1 = new Benchmark;
    $td = timediff($t1, $t0);
    print "the code took:",timestr($td),"<br>\n";

    print $eastring . "<br>";
    print "cookie digest: " . $adigest . "<br>";
    print "<p>expiration: $expdate\n<br>digest: $digest\n";


    if ($digest == $adigest) {
	print "<p>Authentication successful!<br>\n";
    } else {
	print "<meta http-equiv=\"Refresh\" content=\"1; url=http://your.host.here.com/auth1.html\">";
	print "<p>Authentication FAILED!<br>\n";
    }

}

